package net.sourceforge.rapidprops;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import org.junit.Test;
import static org.junit.Assert.*;

public class ParameterizedClassTest {
	public static interface SomeInterface {
		int getSomePrimitiveInt();
		Long getSomeLong();
		List getSomeRawTypeList();
		List<?> getSomeWildcardList();
		List<Double> getSomeDoubleList();
		List<boolean[]> getSomePrimitiveBooleanArrayList();
		List<Boolean[]> getSomeBooleanArrayList();
		List<int[]> getSomePrimitiveIntArrayList();
		List<int[][]> getSomePrimitiveIntIntArrayList();
		List<int[][][]> getSomePrimitiveIntIntIntArrayList();

		Map<float[], Long> getSomePrimitiveFloatArraysLongMap();		
		Map<?, short[]> getSomeWildcardShortArrayMap();		
	}
	
	private static Class<?> getParameterizedClass(String methodName, int parameterIndex) throws SecurityException, NoSuchMethodException {
		Method m = SomeInterface.class.getMethod(methodName);
		try {
			return PropsHelper.getParameterizedClass(m, parameterIndex);
		} catch(RapidPropsException e) {
			return null;
		}			
	}

	@Test
	public void test1() throws SecurityException, NoSuchMethodException {
		assertNull(getParameterizedClass("getSomePrimitiveInt", 0));
		assertNull(getParameterizedClass("getSomeLong", 0));
		assertNull(getParameterizedClass("getSomeRawTypeList", 0));
		assertNull(getParameterizedClass("getSomeWildcardList", 0));
		assertEquals(Double.class, getParameterizedClass("getSomeDoubleList", 0));
		assertNull(getParameterizedClass("getSomeDoubleList", 1));
		assertEquals(boolean[].class, getParameterizedClass("getSomePrimitiveBooleanArrayList", 0));
		assertEquals(Boolean[].class, getParameterizedClass("getSomeBooleanArrayList", 0));
		assertEquals(int[].class, getParameterizedClass("getSomePrimitiveIntArrayList", 0));
		assertNull(getParameterizedClass("getSomePrimitiveIntIntArrayList", 0));
		assertNull(getParameterizedClass("getSomePrimitiveIntIntIntArrayList", 0));
		
		assertEquals(float[].class, getParameterizedClass("getSomePrimitiveFloatArraysLongMap", 0));
		assertEquals(Long.class, getParameterizedClass("getSomePrimitiveFloatArraysLongMap", 1));

		assertNull(getParameterizedClass("getSomeWildcardShortArrayMap", 0));
		assertEquals(short[].class, getParameterizedClass("getSomeWildcardShortArrayMap", 1));
	}
}
